@@ -116,6 +116,7 @@ __version__ = '0.4.5'
 
 import platform
 import logging
+import select
 import time
 import sys
 from functools import partial
@@ -136,31 +137,32 @@ class XMODEM(object):
     XMODEM Protocol handler, expects two callables which encapsulate the read
         and write operations on the underlying stream.
 
-    Example functions for reading and writing to a serial line:
+    Example functions for reading and writing to a serial line using `pyserial <http://pyserial.sourceforge.net/>`_,
 
-    >>> import serial
-    >>> from xmodem import XMODEM
-    >>> ser = serial.Serial('/dev/ttyUSB0', timeout=0) # or whatever you need
-    >>> def getc(size, timeout=1):
-    ...     return ser.read(size) or None
-    ...
-    >>> def putc(data, timeout=1):
-    ...     return ser.write(data) or None
-    ...
-    >>> modem = XMODEM(getc, putc)
+    .. code-block:: python
 
+        import serial
+        from xmodem import XMODEM
+        ser = serial.Serial('/dev/ttyUSB0', timeout=0) # or whatever you need
+        def getc(size, timeout=1):
+            return ser.read(size) or None
+
+        def putc(data, timeout=1):
+            return ser.write(data) or None
+
+        modem = XMODEM(getc, putc)
 
     :param getc: Function to retrieve bytes from a stream. The function takes
         the number of bytes to read from the stream and a timeout in seconds as
         parameters. It must return the bytes which were read, or ``None`` if a
-        timeout occured.
+        timeout occurred.
     :type getc: callable
     :param putc: Function to transmit bytes to a stream. The function takes the
         bytes to be written and a timeout in seconds as parameters. It must
         return the number of bytes written to the stream, or ``None`` in case of
         a timeout.
     :type putc: callable
-    :param mode: XMODEM protocol mode
+    :param mode: Either ``xmodem`` or ``xmodem1k``, defaults to ``xmodem``.
     :type mode: string
     :param pad: Padding character to make the packets match the packet size
     :type pad: char
@@ -281,11 +283,11 @@ class XMODEM(object):
                     if not quiet:
                         print('received CAN', file=sys.stderr)
                     if cancel:
-                        self.log.info('Transmission canceled: received 2xCAN '
+                        self.log.info('Transmission canceled: received CAN CAN '
                                       'at start-sequence')
                         return False
                     else:
-                        self.log.debug('cancellation at start sequence.')
+                        self.log.debug('received CAN at start of sequence.')
                         cancel = 1
                 elif char == EOT:
                     self.log.info('Transmission canceled: received EOT '
@@ -297,7 +299,7 @@ class XMODEM(object):
 
             error_count += 1
             if error_count > retry:
-                self.log.info('send error: error_count reached %d, '
+                self.log.error('send error: error_count reached %d, '
                               'aborting.', retry)
                 self.abort(timeout=timeout)
                 return False
@@ -329,6 +331,8 @@ class XMODEM(object):
                     if callable(callback):
                         callback(total_packets, success_count, error_count)
                     error_count = 0
+                    # keep track of sequence
+                    sequence = (sequence + 1) % 0x100
                     break
 
                 self.log.error('send error: expected ACK; got %r for block %d',
@@ -344,8 +348,7 @@ class XMODEM(object):
                     self.abort(timeout=timeout)
                     return False
 
-            # keep track of sequence
-            sequence = (sequence + 1) % 0x100
+
 
         while True:
             self.log.debug('sending EOT, awaiting ACK')
@@ -359,8 +362,10 @@ class XMODEM(object):
             else:
                 self.log.error('send error: expected ACK; got %r', char)
                 error_count += 1
+                if callback:
+                    callback(total_packets, success_count, error_count)
                 if error_count > retry:
-                    self.log.warn('EOT was not ACKd, aborting transfer')
+                    self.log.warning('EOT was not ACKd, aborting transfer')
                     self.abort(timeout=timeout)
                     return False
 
@@ -387,7 +392,7 @@ class XMODEM(object):
             _bytes.append(crc)
         return bytearray(_bytes)
 
-    def recv(self, stream, crc_mode=1, retry=16, timeout=60, delay=1, quiet=0):
+    def recv(self, stream, crc_mode=1, retry=16, timeout=60, delay=1, quiet=0, callback=None):
         '''
         Receive a stream via the XMODEM protocol.
 
@@ -400,7 +405,7 @@ class XMODEM(object):
 
         :param stream: The stream object to write data to.
         :type stream: stream (file, etc.)
-        :param crc_mode: XMODEM CRC mode
+        :param crc_mode: XMODEM CRC mode, 0 is standard checksum, 1 is 16-bit checksum.
         :type crc_mode: int
         :param retry: The maximum number of times to try to resend a failed
                       packet before failing.
@@ -412,37 +417,46 @@ class XMODEM(object):
         :type delay: int
         :param quiet: If ``True``, write transfer information to stderr.
         :type quiet: bool
+        :param callback: Reference to a callback function that has the
+                         following signature::
 
+                           def callback(total_packets: int, success_count: int, error_count: int, packet_size: int)
+
+                         This is useful for tracking progress state while an xmodem transfer is
+                         underway. As packet size may be negotiated, it also included as the final
+                         argument of the callback. note that the value of error_count resets to 0
+                         after any successful block transfer.
+        :type callback: callable
         '''
 
         # initiate protocol
         error_count = 0
-        char = 0
         cancel = 0
+        empty = 0
         while True:
             # first try CRC mode, if this fails,
             # fall back to checksum mode
             if error_count >= retry:
-                self.log.info('error_count reached %d, aborting.', retry)
+                self.log.error('error_count reached %d, aborting.', retry)
                 self.abort(timeout=timeout)
                 return None
             elif crc_mode and error_count < (retry // 2):
                 if not self.putc(CRC):
-                    self.log.debug('recv error: putc failed, '
-                                   'sleeping for %d', delay)
+                    self.log.warning('recv error: putc failed, '
+                                     'sleeping for %d', delay)
                     time.sleep(delay)
                     error_count += 1
             else:
                 crc_mode = 0
                 if not self.putc(NAK):
-                    self.log.debug('recv error: putc failed, '
-                                   'sleeping for %d', delay)
+                    self.log.warning('recv error: putc failed, '
+                                     'sleeping for %d', delay)
                     time.sleep(delay)
                     error_count += 1
 
             char = self.getc(1, timeout)
             if char is None:
-                self.log.warn('recv error: getc timeout in start sequence')
+                self.log.warning('recv error: getc timeout in start sequence')
                 error_count += 1
                 continue
             elif char == SOH:
@@ -457,10 +471,26 @@ class XMODEM(object):
                                   'at start-sequence')
                     return None
                 else:
-                    self.log.debug('cancellation at start sequence.')
+                    self.log.debug('recv: CAN, cancellation at start sequence')
                     cancel = 1
+            elif char == EOT:
+                if empty:
+                    self.log.info('transmission canceled: file empty')
+                    self.putc(CAN)
+                    self.putc(CAN)
+                    # 'purge' any remaining data on the line
+                    while True:
+                        if self.getc(1, timeout=1) is None:
+                            break
+                        time.sleep(.001) # better cpu usage
+                    return 0
+                else:
+                    self.log.debug('first eot received ')
+                    empty = 1
             else:
                 error_count += 1
+                if callable(callback):
+                    callback(0, 0, error_count, 128)
 
         # read data
         error_count = 0
@@ -468,6 +498,8 @@ class XMODEM(object):
         packet_size = 128
         sequence = 1
         cancel = 0
+        total_packets = 0
+        success_count = 0
         while True:
             while True:
                 if char == SOH:
@@ -486,6 +518,8 @@ class XMODEM(object):
                     self.putc(ACK)
                     self.log.info("Transmission complete, %d bytes",
                                   income_size)
+                    if callable(callback):
+                        callback(total_packets, success_count, error_count, packet_size)
                     return income_size
                 elif char == CAN:
                     # cancel at two consecutive cancels
@@ -501,13 +535,19 @@ class XMODEM(object):
                                'got {0!r}'.format(char))
                     if not quiet:
                         print(err_msg, file=sys.stderr)
-                    self.log.warn(err_msg)
+                    self.log.warning(err_msg)
                     error_count += 1
+                    if callable(callback):
+                        callback(total_packets, success_count, error_count, packet_size)
                     if error_count > retry:
                         self.log.info('error_count reached %d, aborting.',
                                       retry)
                         self.abort()
                         return None
+                    time.sleep(timeout)
+
+                # Reads next char from stream
+                char = self.getc(1, timeout=1)
 
             # read sequence
             error_count = 0
@@ -515,13 +555,13 @@ class XMODEM(object):
             self.log.debug('recv: data block %d', sequence)
             seq1 = self.getc(1, timeout)
             if seq1 is None:
-                self.log.warn('getc failed to get first sequence byte')
+                self.log.warning('getc failed to get first sequence byte')
                 seq2 = None
             else:
                 seq1 = ord(seq1)
                 seq2 = self.getc(1, timeout)
                 if seq2 is None:
-                    self.log.warn('getc failed to get second sequence byte')
+                    self.log.warning('getc failed to get second sequence byte')
                 else:
                     # second byte is the same as first as 1's complement
                     seq2 = 0xff - ord(seq2)
@@ -530,9 +570,10 @@ class XMODEM(object):
                 # consume data anyway ... even though we will discard it,
                 # it is not the sequence we expected!
                 self.log.error('expected sequence %d, '
-                               'got (seq1=%r, seq2=%r), '
-                               'receiving next block, will NAK.',
-                               sequence, seq1, seq2)
+                               'got (seq1=%r, seq2=%r), reading next '
+                               '%d bytes, anyway',
+                               sequence, seq1, seq2,
+                               packet_size + 1 + crc_mode)
                 self.getc(packet_size + 1 + crc_mode)
             else:
                 # sequence is ok, read packet
@@ -542,6 +583,10 @@ class XMODEM(object):
 
                 # valid data, append chunk
                 if valid:
+                    total_packets += 1
+                    success_count += 1
+                    if callable(callback):
+                        callback(total_packets, success_count, error_count, packet_size)
                     income_size += len(data)
                     stream.write(data)
                     self.putc(ACK)
@@ -551,7 +596,7 @@ class XMODEM(object):
                     continue
 
             # something went wrong, request retransmission
-            self.log.warn('recv error: purge, requesting retransmission (NAK)')
+            self.log.warning('recv error: purge, requesting retransmission (NAK)')
             while True:
                 # When the receiver wishes to <nak>, it should call a "PURGE"
                 # subroutine, to wait for the line to clear. Recall the sender
@@ -564,6 +609,9 @@ class XMODEM(object):
                 data = self.getc(1, timeout=1)
                 if data is None:
                     break
+            error_count += 1
+            if callable(callback):
+                callback(total_packets, success_count, error_count, packet_size)
             self.putc(NAK)
             # get next start-of-header byte
             char = self.getc(1, timeout)
@@ -578,7 +626,7 @@ class XMODEM(object):
             our_sum = self.calc_crc(data)
             valid = bool(their_sum == our_sum)
             if not valid:
-                self.log.warn('recv error: checksum fail '
+                self.log.warning('recv error: checksum fail '
                               '(theirs=%04x, ours=%04x), ',
                               their_sum, our_sum)
         else:
@@ -589,7 +637,7 @@ class XMODEM(object):
             our_sum = self.calc_checksum(data)
             valid = their_sum == our_sum
             if not valid:
-                self.log.warn('recv error: checksum fail '
+                self.log.warning('recv error: checksum fail '
                               '(theirs=%02x, ours=%02x)',
                               their_sum, our_sum)
         return valid, data
@@ -599,8 +647,8 @@ class XMODEM(object):
         Calculate the checksum for a given block of data, can also be used to
         update a checksum.
 
-            >>> csum = modem.calc_checksum('hello')
-            >>> csum = modem.calc_checksum('world', csum)
+            >>> csum = modem.calc_checksum(b'hello')
+            >>> csum = modem.calc_checksum(b'world', csum)
             >>> hex(csum)
             '0x3c'
 
@@ -632,7 +680,7 @@ XMODEM1k = partial(XMODEM, mode='xmodem1k')
 
 def _send(mode='xmodem', filename=None, timeout=30):
     '''Send a file (or stdin) using the selected mode.'''
-    
+
     if filename is None:
         si = sys.stdin
     else:
@@ -644,7 +692,7 @@ def _send(mode='xmodem', filename=None, timeout=30):
     def _getc(size, timeout=timeout):
         read_ready, _, _ = select.select([so], [], [], timeout)
         if read_ready:
-            data = stream.read(size)
+            data = si.read(size)
         else:
             data = None
         return data
@@ -666,7 +714,6 @@ def _send(mode='xmodem', filename=None, timeout=30):
 def run():
     '''Run the main entry point for sending and receiving files.'''
     import argparse
-    import serial
     import sys
 
     platform = sys.platform.lower()
@@ -677,16 +724,16 @@ def run():
         default_port = '/dev/ttyS0'
 
     parser = argparse.ArgumentParser()
-    parser.add_argument('-p', '--port', default=default_port,
-                        help='serial port')
-    parser.add_argument('-r', '--rate', default=9600, type=int,
-                        help='baud rate')
-    parser.add_argument('-b', '--bytesize', default=serial.EIGHTBITS,
-                        help='serial port transfer byte size')
-    parser.add_argument('-P', '--parity', default=serial.PARITY_NONE,
-                        help='serial port parity')
-    parser.add_argument('-S', '--stopbits', default=serial.STOPBITS_ONE,
-                        help='serial port stop bits')
+    # parser.add_argument('-p', '--port', default=default_port,
+    #                     help='serial port')
+    # parser.add_argument('-r', '--rate', default=9600, type=int,
+    #                     help='baud rate')
+    # parser.add_argument('-b', '--bytesize', default=serial.EIGHTBITS,
+    #                     help='serial port transfer byte size')
+    # parser.add_argument('-P', '--parity', default=serial.PARITY_NONE,
+    #                     help='serial port parity')
+    # parser.add_argument('-S', '--stopbits', default=serial.STOPBITS_ONE,
+    #                     help='serial port stop bits')
     parser.add_argument('-m', '--mode', default='xmodem',
                         help='XMODEM mode (xmodem, xmodem1k)')
     parser.add_argument('-t', '--timeout', default=30, type=int,
@@ -701,82 +748,11 @@ def run():
                              help='filename to receive, empty sends to stdout')
 
     options = parser.parse_args()
-    
+
     if options.subcommand == 'send':
         return _send(options.mode, options.filename, options.timeout)
-    elif options.subcommand == 'recv':
-        return _recv(options.mode, options.filename, options.timeout)
-
-
-def runx():
-    import optparse
-    import subprocess
-
-    parser = optparse.OptionParser(
-        usage='%prog [<options>] <send|recv> filename filename')
-    parser.add_option('-m', '--mode', default='xmodem',
-                      help='XMODEM mode (xmodem, xmodem1k)')
-
-    options, args = parser.parse_args()
-    if len(args) != 3:
-        parser.error('invalid arguments')
-        return 1
-
-    elif args[0] not in ('send', 'recv'):
-        parser.error('invalid mode')
-        return 1
-
-    def _func(so, si):
-        import select
-
-        print(('si', si))
-        print(('so', so))
-
-        def getc(size, timeout=3):
-            read_ready, _, _ = select.select([so], [], [], timeout)
-            if read_ready:
-                data = so.read(size)
-            else:
-                data = None
-
-            print(('getc(', repr(data), ')'))
-            return data
-
-        def putc(data, timeout=3):
-            _, write_ready, _ = select.select([], [si], [], timeout)
-            if write_ready:
-                si.write(data)
-                si.flush()
-                size = len(data)
-            else:
-                size = None
-
-            print(('putc(', repr(data), repr(size), ')'))
-            return size
-
-        return getc, putc
-
-    def _pipe(*command):
-        pipe = subprocess.Popen(command,
-                                stdout=subprocess.PIPE,
-                                stdin=subprocess.PIPE)
-        return pipe.stdout, pipe.stdin
-
-    if args[0] == 'recv':
-        getc, putc = _func(*_pipe('sz', '--xmodem', args[2]))
-        stream = open(args[1], 'wb')
-        xmodem = XMODEM(getc, putc, mode=options.mode)
-        status = xmodem.recv(stream, retry=8)
-        assert status, ('Transfer failed, status is', False)
-        stream.close()
-
-    elif args[0] == 'send':
-        getc, putc = _func(*_pipe('rz', '--xmodem', args[2]))
-        stream = open(args[1], 'rb')
-        xmodem = XMODEM(getc, putc, mode=options.mode)
-        sent = xmodem.send(stream, retry=8)
-        assert sent is not None, ('Transfer failed, sent is', sent)
-        stream.close()
+    # elif options.subcommand == 'recv':
+    #     return _recv(options.mode, options.filename, options.timeout)
 
 if __name__ == '__main__':
     sys.exit(run())
